/* 
 * Russian hot keys bugfix.
 * Copyright (c) 2006-2007 by Relan (relan@bk.ru)
 * This software is distributed under the GNU General Public License 
 * version 3 or any later version at your choice.
 *
 * Hebrew support added by Muxec (golub_m@netvision.net.il).
 * Cyrillic Dvorak support added by Pavel Fertser (fercerpav@gmail.com).
 *
 * Thanks to all who helped me to develop this extension. :)
 */

// enables debug logging to console
var RhkbfDebug = false;

/*	Cyrillic codes translation table.
	Capital letters are translated to lower case to fix hot keys with 
	CAPS LOCK on. Actually the following translation is performed:
	tr/АБВГДЕЖЗИЙКЛМНОПРСТУФХЦЧШЩЪЫЬЭЮЯ/F,DULT;PBQRKVYJGHCNEA[WXIO]SM'.Z/
	tr/абвгдежзийклмнопрстуфхцчшщъыьэюя/f,dult;pbqrkvyjghcnea[wxio]sm'.z/
*/
var RhkbTableCyr = new Array(
	// А-Я => a-z (0x0410-0x042F)
	0x66, 0x2C, 0x64, 0x75, 0x6C, 0x74, 0x3B, 0x70, 
	0x62, 0x71, 0x72, 0x6B, 0x76, 0x79, 0x6A, 0x67, 
	0x68, 0x63, 0x6E, 0x65, 0x61, 0x5B, 0x77, 0x78, 
	0x69, 0x6F, 0x5D, 0x73, 0x6D, 0x27, 0x2E, 0x7A,
	// а-я => a-z (0x0430-0x044F)
	0x66, 0x2C, 0x64, 0x75, 0x6C, 0x74, 0x3B, 0x70, 
	0x62, 0x71, 0x72, 0x6B, 0x76, 0x79, 0x6A, 0x67, 
	0x68, 0x63, 0x6E, 0x65, 0x61, 0x5B, 0x77, 0x78, 
	0x69, 0x6F, 0x5D, 0x73, 0x6D, 0x27, 0x2E, 0x7A
);


/*	Hebrew codes translation table. This code was sent to me by Muxec.
	In Hebrew there are no capital letters at all. No need to take care
	or CAPS LOCK mode. I don't really know how this table works, so ask 
	Muxec if you find a bug in it. :)
*/
var RhkbTableHeb = new Array(
	// א-ת => a-z (0x05D0-0x05EA)
	0x74, 0x63, 0x64, 0x73, 0x76, 0x75, 0x7A, 0x6A, 
	0x79, 0x68, 0x6C, 0x66, 0x6B, 0x6F, 0x6E, 0x69, 
	0x62, 0x78, 0x67, 0x3B, 0x70, 0x2E, 0x6D, 0x65, 
	0x72, 0x61, 0x2C
);


/*	Cyrillic codes translation table for Dvorak layout. This code was
	sent to me by Pavel.
*/
var RhkbTableCyrDvorak = new Array(
	// А-Я => a-z (0x0410-0x042F)
	0x75, 0x77, 0x65, 0x67, 0x6E, 0x79, 0x73, 0x6C, 
	0x78, 0x27, 0x70, 0x74, 0x6B, 0x66, 0x68, 0x69, 
	0x64, 0x6A, 0x62, 0x2E, 0x61, 0x2F, 0x2C, 0x71, 
	0x63, 0x72, 0x3D, 0x6F, 0x6D, 0x2D, 0x76, 0x3B,
	// а-я => a-z (0x0430-0x044F)
	0x75, 0x77, 0x65, 0x67, 0x6E, 0x79, 0x73, 0x6C, 
	0x78, 0x27, 0x70, 0x74, 0x6B, 0x66, 0x68, 0x69, 
	0x64, 0x6A, 0x62, 0x2E, 0x61, 0x2F, 0x2C, 0x71, 
	0x63, 0x72, 0x3D, 0x6F, 0x6D, 0x2D, 0x76, 0x3B);


var Rhkbf = {
	log: function(message) {
		if (RhkbfDebug)
		{
			var consrv = Components.classes["@mozilla.org/consoleservice;1"]
				.getService(Components.interfaces.nsIConsoleService);
			consrv.logStringMessage("Rhkbf: " + message + ".");
		}
	},

	getLayout: function() {
		var layout = 0;	// default is Cyrillic
		// get keyboard layout preference
		try {
			var prefs = Components
				.classes["@mozilla.org/preferences-service;1"]
				.getService(Components.interfaces.nsIPrefService);
			prefs = prefs.getBranch("extensions.rhkbf.");
			layout = prefs.getIntPref("layout");
			Rhkbf.log("chosen layout: " + layout);
		} catch(ex) {
			/*	Default preferences file probably does not exist.
				It's a common case for Mozilla Suite and SeaMonkey
				because they do not copy default/preferences/rhkbf.js
				as Firefox and Thunderbird do. User has to add the 
				corresponding preference ("extensions.rhkbf.layout")
				manually if he/she wants to alter the default fallback
				layout (which is Russian "йцукен").
			*/
			Rhkbf.log("failed to get layout, defaulting to " + layout);
		}
		return layout;
	},

	/*	This method suits well for Russian and Ukrainian "йцукен" layouts, 
		but not for Belorussian as it conflicts with Ukrainian. Nevertheless 
		nobody asked me for Belorussian support, so I don't care.
		
		For Hebrew there is a bug: hot keys with "Q" and "W" are not 
		translated because it is impossible to distinguish them from 
		"/" and "'" respectively. I think is's better to do nothing with
		this bug than break hot keys behaviour is US layout.
	*/
	translateCommon: function(code) {
		// look up in Cyrillic table.
		if (code >= 0x0410 /* А */ && code <= 0x044F /* я */)
			return RhkbTableCyr[code - 0x0410];
		// some Ukrainian characters are outside Cyrillic table bounds
		switch (code) {
		case 0x0406: return 0x53; // І - S
		case 0x0456: return 0x73; // і - s
		case 0x0407: return 0x7D; // Ї - }
		case 0x0457: return 0x5D; // ї - ]
		case 0x0404: return 0x22; // Є - "
		case 0x0454: return 0x27; // є - '
		case 0x0490: return 0x7C; // Ґ - |
		case 0x0491: return 0x5C; // ґ - \
		}
		// look up in Hebrew table
		if (code >= 0x05D0 /* א */ && code <= 0x05EA /* ת */)
			return RhkbTableHeb[code - 0x05D0];
		// unrecognized
		return 0;
	},

	/*	Alternative translation method for Dvorak layout. Only Russian and
		Ukrainian are supported.
	*/
	translateDvorak: function(code) {
		// look up in Cyrillic table.
		if (code >= 0x0410 /* А */ && code <= 0x044F /* я */)
			return RhkbTableCyrDvorak[code - 0x0410];
		// some Ukrainian characters are outside Cyrillic table bounds
		switch (code) {
		case 0x0406: return 0x4F; // І - O
		case 0x0456: return 0x6F; // і - o
		case 0x0407: return 0x2B; // Ї - +
		case 0x0457: return 0x3D; // ї - =
		case 0x0404: return 0x5F; // Є - _
		case 0x0454: return 0x2D; // є - -
		case 0x0490: return 0x7C; // Ґ - |
		case 0x0491: return 0x5C; // ґ - \
		}
		// unrecognized
		return 0;
	},

	translate: function(code) {
		var layout = Rhkbf.getLayout();
		switch (layout) {
		case 0: return Rhkbf.translateCommon(code);
		case 2: return Rhkbf.translateDvorak(code);
		default:	// user specified wrong layout
			Rhkbf.log("bad keyboard layout: " + layout);
			return 0;
		}
	},

	keypress: function(evt) {
		Rhkbf.log("key " + evt.charCode + " pressed, modifiers:" 
			+ (evt.ctrlKey  ? " ctrl"  : "")
			+ (evt.altKey   ? " alt"   : "")
			+ (evt.shiftKey ? " shift" : ""));
		if (!evt.altKey && !evt.ctrlKey)
			return;
		var trcode = Rhkbf.translate(evt.charCode);
		if (trcode == 0)	// unrecognized
		{
			Rhkbf.log("unrecognized char code " 
				+ evt.charCode + ", skipping");
			return;
		}
		Rhkbf.log("translating " + evt.charCode + " to " + trcode);
		var trevt = document.createEvent("KeyboardEvent");
		trevt.initKeyEvent(evt.type, evt.bubbles, evt.cancelable, 
			evt.view, evt.ctrlKey, evt.altKey, evt.shiftKey, evt.metaKey, 
			evt.keyCode, trcode);
		evt.preventDefault();
		evt.target.dispatchEvent(trevt);
	},

	load: function() {
		try {
			window.removeEventListener("keypress", Rhkbf.keypress, true);
		} catch(ex) {
			Rhkbf.log("failed to remove event listener.");
		}
		window.addEventListener("keypress", Rhkbf.keypress, true);

	}
}

window.addEventListener("load", function() { Rhkbf.load(); }, false); 
